// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "globals.hh"
#include "PingusResource.hh"

#include "OptionMenu.hh"

// Define the global option menu
OptionMenu   option_menu;

// ----- OptionEntry -----

OptionEntry::OptionEntry(string s, bool* v, int x, int y)
{
  font = CL_Res_Font::load("Fonts/smallfont_h",  *PingusResource::get("global.dat"));
  str = s;
  value_bool = v;
  value_int = 0;
  value_str = 0;
  x_pos = x;
  y_pos = y;
}

OptionEntry::OptionEntry(string s, string* v, int x, int y)
{
  font = CL_Res_Font::load("Fonts/smallfont_h",  *PingusResource::get("global.dat"));
  str = s;
  value_str = v;
  value_int = 0;
  value_bool = 0;
  x_pos = x;
  y_pos = y;
}

OptionEntry::OptionEntry(string s, int* v, int x, int y)
{
  font = CL_Res_Font::load("Fonts/smallfont_h",  *PingusResource::get("global.dat"));
  str = s;
  value_str = 0;
  value_int = v;
  value_bool = 0;
  x_pos = x;
  y_pos = y;
}

void
OptionEntry::draw()
{
  if (!mouse_over()) {
    CL_Display::fill_rect(x_pos - 7, y_pos - 1,
			  x_pos + CL_Display::get_width() - 35, y_pos + 11,
			  0.5, 0.5, 0.5, 0.5);
  } else {
    CL_Display::fill_rect(x_pos - 7, y_pos - 1,
			  x_pos + CL_Display::get_width() - 35, y_pos + 11,
			  1.0, 1.0, 1.0, 0.5);
  }
  font->print_left(x_pos, y_pos, str.c_str());

  if (value_bool) {
    if (*value_bool) {
      font->print_right(x_pos + CL_Display::get_width() - 35, y_pos, "on");
    } else {
      font->print_right(x_pos + CL_Display::get_width() - 35, y_pos, "off");
    }
  } else if (value_str) {
    font->print_right(x_pos + CL_Display::get_width() - 35, y_pos, value_str->c_str());    
  } else if (value_int) {
    char str[256];
    sprintf(str, "%d", *value_int);
    font->print_right(x_pos + CL_Display::get_width() - 35, y_pos, str);    
  }
}

void
OptionEntry::toggle()
{
  if (value_bool) {
    *(value_bool) = !*(value_bool);
  } else if (value_str) {
    cout << "Not implemented" << endl;
  } else if (value_int) {
    ++(*value_int);
  }
}

void
OptionEntry::rtoggle()
{
  if (value_bool) {
    *(value_bool) = !*(value_bool);
  } else if (value_str) {
    cout << "Not implemented" << endl;
  } else if (value_int) {
    --(*value_int);    
  }
}

bool
OptionEntry::mouse_over()
{
  if (CL_Mouse::get_x() > x_pos && CL_Mouse::get_x() < x_pos + CL_Display::get_width() - 30
      && CL_Mouse::get_y() > y_pos && CL_Mouse::get_y() < y_pos + 15) {
    return true;
  } else {
    return false;
  }
}

// ----- OptionMenu -----

OptionMenu::OptionMenu()
{
  is_init = false;
}

OptionMenu::~OptionMenu()
{
  
}

void
OptionMenu::init() 
{
  font = CL_Res_Font::load("Fonts/smallfont_h",  *PingusResource::get("global.dat"));
  title_font = CL_Res_Font::load("Fonts/clanfont", *PingusResource::get("global.dat"));
  background =  CL_Res_Surface::load("Textures/rocktile", *PingusResource::get("global.dat"));
  entry_x = 20;
  entry_y = 60;
  is_init = true;

  // Bool options
  add_entry("Gimmicks",  &gimmicks_enabled);
  add_entry("Unlimited Actions",  &unlimited_actions);
  add_entry("Print fps", &print_fps);
  add_entry("Debug actions", &debug_actions);  
  add_entry("Cursor enabled", &cursor_enabled);  
  add_entry("Fast Mode", &fast_mode);
  add_entry("Pingus Datadir", &pingus_datadir);
  add_entry("Game Speed", &game_speed);
  add_entry("Verbose", &verbose);
  add_entry("Music", &music_enabled);
  add_entry("Sound", &sound_enabled);
  add_entry("Level Previews", &previews_enabled);
  add_entry("Force Vectors", &force_vectors);
}

void
OptionMenu::add_entry(string e, bool* v)
{
  entry.push_back(OptionEntry(e, v, entry_x, entry_y));
  entry_y += 20;
}

void 
OptionMenu::add_entry(string e, int* v)
{
  entry.push_back(OptionEntry(e, v, entry_x, entry_y));
  entry_y += 20;
}

void 
OptionMenu::add_entry(string e, string* v)
{
  entry.push_back(OptionEntry(e, v, entry_x, entry_y));
  entry_y += 20;
}

void
OptionMenu::draw_background()
{
  for(int y=0; y < CL_Display::get_height(); y += background->get_height()) {
    for(int x=0; x < CL_Display::get_width(); x += background->get_width()) {
      background->put_screen(x, y);
    }
  }
}

void
OptionMenu::draw()
{  
  draw_background();
  
  title_font->print_center(CL_Display::get_width() / 2, 10, "Pingus Option Menu");
  for(int i=0; i < entry.size(); ++i) {
    entry[i].draw();
  }
}

void
OptionMenu::check_click()
{
  for(int i=0; i < entry.size(); ++i) {
    if (entry[i].mouse_over()) {
      entry[i].toggle();
      break;
    }
  }
}

void
OptionMenu::display()
{
  if (!is_init)
    init();

  int temp_index = -2;
  int index = -2;

  while(CL_Mouse::left_pressed()) {
    CL_System::keep_alive();
  }

  while(!CL_Mouse::middle_pressed()
	&& !CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE)) {
    index = current_index();
    if (temp_index != index)
      {
	draw();
	CL_Display::flip_display();
	temp_index = index;
      }
    if (CL_Mouse::left_pressed()) {
      if (index >= 0) 
	entry[index].toggle();
      draw();
      CL_Display::flip_display();
      while(CL_Mouse::left_pressed())
	CL_System::keep_alive();
    }
    if (CL_Mouse::right_pressed()) {
      if (index >= 0) 
	entry[index].rtoggle();
      draw();
      CL_Display::flip_display();
      while(CL_Mouse::right_pressed())
	CL_System::keep_alive();
    }
    CL_System::keep_alive();
  }
  while(CL_Mouse::middle_pressed()) {
    CL_System::keep_alive();
  }
  while(CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE)) {
    CL_System::keep_alive();
  }
}

int
OptionMenu::current_index()
{
  for(int i=0; i < entry.size(); ++i) {
    if (entry[i].mouse_over()) {
      return i;
    }
  }
  return -1;
}

/* EOF */
