// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <iostream>

#include "globals.hh"
#include "AlphaButton.hh"
#include "PingusGame.hh"
#include "FileSelector.hh"
#include "PingusMenu.hh"

PingusMenu::PingusMenu()
{
  int a = CL_Display::get_width() / 3;

  bg    = CL_Res_Surface::load("Game/logo", *resource());
  start = CL_Res_Surface::load("Buttons/load", *resource());
  load  = CL_Res_Surface::load("Buttons/load", *resource());
  quit  = CL_Res_Surface::load("Buttons/load", *resource());  
  highlight = CL_Res_Surface::load("Hotspots/star", *resource());
 
  start_x = (a / 2) - (start->get_width() / 2);
  start_y = 200;
  
  load_x = (int)((1.5 * a) - (load->get_width() / 2));
  load_y = 275;
  
  quit_x = 2 * a + (a / 2) - (quit->get_width() / 2);
  quit_y = 200;

  button.push_back(new AlphaButton("Quit", 260, 200, 60, 40));
  button.push_back(new AlphaButton("Play", 260, 240, 60, 40));
  button.push_back(new AlphaButton("Load", 320, 200, 60, 40));
  button.push_back(new AlphaButton("Theme", 320, 240, 60, 40));
  button.push_back(new AlphaButton("Options", 320, 280, 60, 40));

  do_quit = false;
}

PingusMenu::~PingusMenu()
{
}

void
PingusMenu::draw(void)
{
  bg->put_screen(0, 0);

  CL_Display::fill_rect(250, 190, 390, 330,
			0.0, 0.0, 0.0, 0.3);

  for(int i=0; i < button.size(); ++i) {
    button[i]->draw();
  }

  CL_Display::flip_display();
  CL_System::sleep(10);
}

void
PingusMenu::select(void)
{
  cout << "PingusMenu::select" << endl;

  if (quick_play) {
    do_button(1);
    quick_play = false;
  }

  while(!do_quit) {
    CL_System::keep_alive();
    draw();

    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE)) {
      do_button(0);
    }
    for(int i=0; i < button.size(); ++i) {
      if (button[i]->is_pressed()) {
	do_button(i);
      }
    }
  }
}

void
PingusMenu::do_button(int i)
{
  switch(i) {
  case 0:
    do_quit = true;
    cout << "PingusMenu: Quitting..." << endl;
    break;
  case 1: // Quick Play
    {
      PingusGame game;
      game.start(/* lastlevel */);
    }
    break;
  case 2:
    {
      string levelfile;
      levelfile = file.select(pingus_datadir, "*.plf");
      if (!levelfile.empty()) {
	PingusGame game;
	game.start(levelfile);
      }
    }
    break;
  case 3:
    {
      theme_selector.select();
    }
    break;
  case 4:
    cout << "Option Menu" << endl;
    option_menu.display();
    break;
  default:
    cout << "Not implemented" << endl;
    break;
  }
}

/* EOF */
